
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: _G, ChatFrame1Tab, ChatFrame2Tab, ChatFrame3Tab, ChatFrame4Tab, ChatFrame5Tab, ChatFrame6Tab, ChatFrame7Tab, ChatFrame8Tab, ChatFrame9Tab, ChatFrame10Tab, coroutine, LoggingChat, next, NUM_CHAT_WINDOWS

local CHAT_FRAME_TAB_ALERTING_NOMOUSE_ALPHA = CHAT_FRAME_TAB_ALERTING_NOMOUSE_ALPHA
local SixUI_DisplayGroup1 = SixUI_DisplayGroup1
local SixUI_EditBox = SixUI_EditBox

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(select(2, ...)) -- AddOn, Border, Constants, Database
local M = A:NewModule("Chat Frame")

-------------------------------------------------------------------------------
-- Module
-------------------------------------------------------------------------------

function M:OnEnable()
	self:CreateHooks()
	self:RegisterFrames()
	self:RegisterMessage("SIXUI_CONFIG_UPDATE", "SetFonts")
	self:RegisterMessage("SIXUI_PROFILE_UPDATE", "SetFonts")
	self:ConfigureChatFrames()
	self:ConfigureChatTabs()
	self:EnableChatLogging()
end

function M:CreateHooks()
	if not self:IsHooked("ChatEdit_OnEditFocusGained") then
		self:SecureHook("ChatEdit_OnEditFocusGained")
	end

	if not self:IsHooked("ChatEdit_OnEditFocusLost") then
		self:SecureHook("ChatEdit_OnEditFocusLost")
	end

	if not self:IsHooked("ChatEdit_UpdateHeader") then
		self:SecureHook("ChatEdit_UpdateHeader")
	end

	if not self:IsHooked("CreateFrame") then
		self:SecureHook("CreateFrame")
	end

	if not self:IsHooked("FCF_DockFrame") then
		self:SecureHook("FCF_DockFrame")
	end

	if not self:IsHooked("FCF_UnDockFrame") then
		self:SecureHook("FCF_UnDockFrame")
	end
end

function M:RegisterFrames()
	A:RegisterFrame("ChatFrameMenuButton"):Hide()
	A:RegisterFrame("ChatFrameChannelButton"):SetParent("ChatFrame1Tab")
	A:RegisterFrame("ChatFrameToggleVoiceDeafenButton"):SetParent("ChatFrame1Tab")
	A:RegisterFrame("ChatFrameToggleVoiceMuteButton"):SetParent("ChatFrame1Tab")
	A:RegisterFrame("CombatLogQuickButtonFrame_Custom"):SetPoint("BOTTOMLEFT", "ChatFrame2", "TOPLEFT"):SetPoint("BOTTOMRIGHT", "ChatFrame2", "TOPRIGHT")
	A:RegisterFrame("QuickJoinToastButton"):Hide()
end

-- Iterate over all chat frames.
function M:IterateChatFrames()
	return coroutine.wrap(function ()
		for index = 1, NUM_CHAT_WINDOWS do
			local frameName = "ChatFrame" .. index
			local chatFrame = _G[frameName]

			if chatFrame then
				coroutine.yield(chatFrame, frameName)
			end
		end
	end)
end

-- Configure all chat frames.
function M:ConfigureChatFrames()
	for chatFrame, frameName in self:IterateChatFrames() do
		self:ConfigureChatFrame(frameName, chatFrame)
	end
end

-- Configure a chat frame, including its textures, hooks and edit box.
function M:ConfigureChatFrame(frameName, chatFrame)
	local chatTab = _G[frameName .. "Tab"]
	local resizeButton = _G[frameName .. "ResizeButton"]

	chatFrame:DisableDrawLayer("BACKGROUND")
	chatFrame:DisableDrawLayer("BORDER")
	chatFrame:SetClampedToScreen(false)
	chatFrame:SetFading(false)
	chatFrame:SetFont(C.TEXT_MEDIUM:GetFont())

	if chatTab then
		self:ConfigureChatTab(chatTab)
	end

	if resizeButton then
		resizeButton:ClearAllPoints()
		resizeButton:SetPoint("BOTTOMRIGHT", chatFrame, "BOTTOMRIGHT", C.CHATFRAME_INSET, -C.CHATFRAME_INSET)
	end

	if not chatFrame.isDocked then
		self:FCF_UnDockFrame(chatFrame)
	end

	if chatFrame.buttonFrame then
		chatFrame.buttonFrame:SetScript("OnShow", chatFrame.buttonFrame.Hide)
		chatFrame.buttonFrame:Hide()
	end

	if chatFrame.ScrollBar then
		chatFrame.ScrollBar:SetScript("OnShow", chatFrame.ScrollBar.Hide)
		chatFrame.ScrollBar:Hide()
	end

	if chatFrame.ScrollToBottomButton then
		chatFrame.ScrollToBottomButton:SetScript("OnShow", chatFrame.ScrollToBottomButton.Hide)
		chatFrame.ScrollToBottomButton:Hide()
	end

	if chatFrame.editBox then
		self:ConfigureEditBox(chatFrame.editBox, chatFrame.editBox:GetName())
	end
end

-- Configure all chat tabs.
function M:ConfigureChatTabs()
	for _, chatTab in next, {
		ChatFrame1Tab,
		ChatFrame2Tab,
		ChatFrame3Tab,
		ChatFrame4Tab,
		ChatFrame5Tab,
		ChatFrame6Tab,
		ChatFrame7Tab,
		ChatFrame8Tab,
		ChatFrame9Tab,
		ChatFrame10Tab,
	} do
		self:ConfigureChatTab(chatTab)
	end
end

-- Configure a chat tab.
function M:ConfigureChatTab(chatTab)
	if not self:IsHooked(chatTab, "SetAlpha") then
		self:SecureHook(chatTab, "SetAlpha", "UpdateChatTab")
	end

	self:UpdateChatTab(chatTab, chatTab:GetAlpha())
end

-- Configure an edit box, including its position and textures.
function M:ConfigureEditBox(editBox, frameName)
	editBox:SetParent(SixUI_EditBox)
	editBox:SetAllPoints(SixUI_EditBox)
	editBox:SetFontObject(C.TEXT_MEDIUM)
	editBox.header:SetFontObject(C.TEXT_MEDIUM)
	editBox:EnableMouse(false)
	editBox:SetAltArrowKeyMode(false)
	editBox:DisableDrawLayer("BACKGROUND")
	editBox:DisableDrawLayer("BORDER")
end

-- Enable chat logging.
function M:EnableChatLogging()
	if not LoggingChat() then
		LoggingChat(true)
	end
end

-- Set the font for all chat frames.
function M:SetFonts()
	for chatFrame in self:IterateChatFrames() do
		chatFrame:SetFont(C.TEXT_MEDIUM:GetFont())
	end
end

-- Hide a chat tab if the mouse is not over it and it's not alerting.
function M:UpdateChatTab(chatTab, alpha)
	if alpha ~= 0 and alpha ~= chatTab.mouseOverAlpha and alpha ~= CHAT_FRAME_TAB_ALERTING_NOMOUSE_ALPHA then
		chatTab:SetAlpha(0)
	end
end

-------------------------------------------------------------------------------
-- Hooks
-------------------------------------------------------------------------------

-- Use a hook to set up the edit box when it gains focus.
function M:ChatEdit_OnEditFocusGained(editBox)
	editBox:EnableMouse(true)
	SixUI_EditBox:SetAlpha(1)
	SixUI_DisplayGroup1:Hide()
end

-- Use a hook to restore the edit box when it loses focus.
function M:ChatEdit_OnEditFocusLost(editBox)
	editBox:EnableMouse(false)
	SixUI_EditBox:SetAlpha(0)
	SixUI_DisplayGroup1:Show()
end

-- Use a hook to synchronise SixUI's border color with the edit box.
function M:ChatEdit_UpdateHeader(editBox)
	SixUI_EditBox:GetBorder():SetColor(editBox:GetTextColor())
end

-- Use a CreateFrame hook to find newly created frames.
function M:CreateFrame(frameType, frameName, parentFrame, inheritsFrame)
	if frameName and frameName:match("^ChatFrame[0-9]+$") then
		self:ConfigureChatFrame(frameName, _G[frameName])
	end
end

function M:FCF_DockFrame(chatFrame)
	B:RemoveBorder(chatFrame)
end

function M:FCF_UnDockFrame(chatFrame)
	B:CreateBorder(chatFrame, C.CHATFRAME_INSET)
end
